#include "BuildSession.h"

#include "Logger.h"
#include "Process.h"
#include "Directory.h"

BuildSession::BuildSession(const String& source_dir, 
    const String& build_dir)
    : m_source_dir(source_dir)
{
    if (build_dir.empty())
    {
        m_build_dir = FileSystemPath::current_dir().value();
    }
    else
    {
        m_build_dir = build_dir;
    }
    m_compiler_flags.push_back("-g");
    m_compiler_flags.push_back("-fno-exceptions");
    m_compiler_flags.push_back("-fno-rtti");
}

Status BuildSession::scan()
{
    LOG_INFO("Scanning sources at:" << m_source_dir);
    Vector<FileSystemPath> toml_files;
    STATUS_CHECK(Directory::getFilesWithExtension(
        m_source_dir, 
        ".toml", 
        toml_files,
        true), "Error looking for build files");

    for(const auto& toml_file : toml_files)
    {
        if (toml_file.file_name() == "build")
        {
            STATUS_CHECK(add_library(toml_file),
                "Error adding library");
        }
    }
    return {};

}

Status BuildSession::add_library(const FileSystemPath& config_path)
{
    LOG_INFO("Adding library at: " << config_path);
    BuildLibrary lib(config_path);
    STATUS_CHECK(lib.scan(), "Error scanning library");
    m_libraries.push_back(lib);
    return {};
}

Status BuildSession::build()
{
    for(const auto& library : m_libraries)
    {
        for(const auto& source : library.get_sources())
        {
            Vector<String> args = m_compiler_flags;
            args.push_back("-c");
            for(const auto& include_dir : library.get_include_dirs())
            {
                args.push_back(_s("-I") + include_dir.str());
            }
            args.push_back(source.str());
            LOG_INFO("Compiling " << source.file_name());
            const auto run_status = Process::launch(m_compiler_command, args);
            if (!run_status.ok())
            {
                return run_status;
            }   
            //break;
        }
        //break;
    }
    return {};
}