#pragma once

#include "Vector.h"
#include "Pair.h"
#include "Index.h"
#include "ByteTypes.h"

class String
{
public:
    String();

    String(const Vector<Byte>& data);

    String(const char* data);

    static String fmt(const char* fmt, ...);

    void append(const Vector<Byte>& data);

    const Vector<char>& data() const;

    bool empty() const;

    const char* raw() const;

    Pair<String, String> rsplit(char c) const;

    Index rindex(char c) const;

    void reverse();

    std::size_t size() const;

    bool slice(std::size_t idx, String& out) const;

    bool slice(std::size_t start, std::size_t end, String& out) const;

    static String to_string(size_t input);

    char operator[](std::size_t idx) const;

    String& operator<<(const String& body);

    String& operator<<(const char* body);

    String& operator<<(size_t idx); 

    template<typename T>
    String& operator<<(const T& stringable)
    {
        *this += stringable.str();
        return *this;
    } 

    String& operator+=(const String& str);

    String& operator+=(char c);

    String operator+(const String& str) const;

    bool operator==(const String& other) const;

    bool operator!=(const String& other) const;

private:
    void append(const char* data);

    Vector<char> m_data;
};

using _s = String;