#pragma once

#include "FileFormats.h"
#include "FileSystemPath.h"
#include "Optional.h"
#include "String.h"
#include "ByteTypes.h"
#include "Pointer.h"

class FileImpl;

class File
{
public:
    enum class AccessMode{
        Read,
        Write,
        ReadWrite
    };

public:
    File(const FileSystemPath& path);

    ~File();

    Status close();

    String dumpBinary();

    String getExtension() const;

    FileFormat::Format inferFormat() const;

    Status readText(String& buffer);

    Status readLines(Vector<String>& lines);

    Status read(String& buffer);

    bool exists() const;

    Status open(AccessMode mode, bool binary = false);

    Status readBinary(VecBytes& bytes);

    Optional<Byte> readNextByte();

    Status writeText(const String& text);

private:
    Ptr<FileImpl> m_impl;
    FileSystemPath m_path;
};
