#include "FileLogger.h"

#include <time.h>

FileLogger::FileLogger()
    :mWorkDirectory(),
        mFileName("MT_Log.txt")
{

}


FileLogger& FileLogger::GetInstance()
{
    static FileLogger instance;
    return instance;
}

FileLogger::~FileLogger()
{

}

void FileLogger::SetWorkDirectory(const String& workDir)
{
    mWorkDirectory = workDir;
}

void FileLogger::SetFileName(const String& fileName)
{
    mFileName = fileName;
}

void FileLogger::Open()
{
    if (mWorkDirectory.empty())
    {
        mWorkDirectory = std::filesystem::current_path().string();
    }
    mFileStream.open(mWorkDirectory + "/" + mFileName);
}

void FileLogger::Close()
{
    mFileStream.close();
}

void FileLogger::LogLine(const std::ostringstream& line)
{
    if (mDisabled)
    {
        return;
    }

    if (!mFileStream.is_open())
    {
        Open();
    }
    mFileStream << line.str() << std::endl;
}

void FileLogger::disable()
{
    mDisabled = true;
}

void FileLogger::LogLine(const String& logType, const std::ostringstream& line, const String& fileName, const String& functionName, int lineNumber)
{
    if (mDisabled)
    {
        return;
    }

    std::time_t t = std::time(nullptr);
    const String cleanedFileName = fileName.substr(fileName.find_last_of("/\\") + 1);
    std::tm time_buf{ };
#ifdef WIN32
    gmtime_s(&time_buf, &t);
#else
    gmtime_r(&t, &time_buf);
#endif
    std::cout << logType << "|" << std::put_time(&time_buf, "%T") << "|" << cleanedFileName << "::" << functionName << "::" << lineNumber << "|" << line.str() << std::endl;
    mFileStream << logType << "|" << std::put_time(&time_buf, "%T") << "|" << cleanedFileName << "::" << functionName << "::" << lineNumber << "|" << line.str() <<  std::endl;
}
