#include "Logger.h"

#include "Time.h"
#include "ConsoleLogger.h"

static Ptr<Logger> s_logger; 

Logger* Logger::get_instance()
{
    if (s_logger.get() == nullptr)
    {
        s_logger = Ptr<ConsoleLogger>::create();
    }
    return s_logger.get();
}

String Logger::build_log_message(Level level, 
    const String& msg, 
    const String& fileName, 
    const String& functionName, 
    int lineNumber)
{
    String log_msg;
    if (level == Level::INFO)
    {
        log_msg += "Info|";
    }
    else if (level == Level::ERROR)
    {
        log_msg += "Error|";
    }
    log_msg += Time::get_now_str() + "|";

    String cleaned_filename;
    if (auto index = fileName.rindex('/'); index.valid())
    {
        fileName.slice(index.value()+1, fileName.size(), cleaned_filename);
    }
    else
    {
        cleaned_filename = fileName;
    }
    log_msg += cleaned_filename + "::";
    log_msg += functionName + "::";
    log_msg += String::to_string(lineNumber) + "|";
    log_msg += msg; 
    return log_msg;
}