#pragma once

#include "Allocator.h"
#include <utility>

template<typename T>
class Ptr
{
public:
    Ptr()
    {
    }

    static Ptr create()
    {
        Ptr p;
        p.allocate();
        return std::move(p);
    }

    void allocate()
    {
        m_raw = m_allocator.allocate();
    }

    ~Ptr()
    {
        if (m_raw != nullptr)
        {
            m_allocator.do_delete(m_raw);
        }
    }

    Ptr(const Ptr& other) = delete;

    Ptr(Ptr&& other)
    {
        *this = std::move(other);
    }

    Ptr<T>& operator=(const Ptr<T>& other) = delete;

    template<typename U>
    Ptr<T>& operator=(Ptr<U>&& other)
    {
        if (this->m_raw != other.get())
        {
            this->m_raw = dynamic_cast<T*>(other.get());
            other.clear_raw();
        }
        return *this;
    }

    T* get()
    {
        return m_raw;
    }

    const T* get() const 
    {
        return m_raw;
    }

    T* operator->()
    {
        return m_raw;
    }

    void clear_raw()
    {
        m_raw = nullptr;
    }

private:
    Allocator<T> m_allocator;
    T* m_raw{nullptr};
};