#pragma once

#include "File.h"

#include <filesystem>
#include "Memory.h"
#include <unordered_map>
#include "String.h"

using Path = std::filesystem::path;

class TomlTable
{
public:
    using Comment = std::pair<unsigned, String>;
    using KeyValuePairs = std::unordered_map<String, String>;

    TomlTable(const String& header);

    void addComment(const Comment& comment);

    void addTable(Ptr<TomlTable> table);

    void addKeyValuePair(const String& key, const String& value);

    String getHeader() const;

    TomlTable* getTable(const String& path);

    KeyValuePairs getKeyValuePairs() const;

private:
    String mHeader;
    std::unordered_map<String, Ptr<TomlTable> > mTables;
    KeyValuePairs mMap;
    Vector<Comment> mComments;
};

class TomlContent
{
public:
    TomlContent();

    TomlTable* getRootTable() const;

    TomlTable* getTable(const String& path) const;

private:
    Ptr<TomlTable> mRootTable;
};

class TomlReader
{
public:
    TomlReader();

    TomlContent* getContent() const;

    void read(const Path& input_path);

    void processLine(const String& line);

    void onHeader(const String& header);

    void onKeyValuePair(const String key, const String value);

private:
    unsigned mLastSectionOffset{ 0 };
    Ptr<TomlContent> mContent;
    TomlTable* mWorkingTable{nullptr};
};
