#pragma once

#include "TypedGrid.h"

template<typename T>
class Grid : public TypedGrid<T>
{
public:
    Grid(const Bounds& bounds, std::size_t numPointsX, std::size_t numPointsY, std::size_t numPointsZ = 1)
        : TypedGrid<T>(bounds, numPointsX, numPointsY, numPointsZ)
    {
        this->mData->initializeTo(numPointsX * numPointsY * numPointsZ, T());
    }

    T getItem(std::size_t idx, std::size_t jdx, std::size_t kdx = 1) const override
    {
        return this->mData->getItem(getOffset(idx, jdx, kdx));
    }

    std::size_t getOffset(std::size_t idx, std::size_t jdx, std::size_t kdx) const
    {
        return idx * this->mNumZ + jdx * this->mNumX * this->mNumZ + kdx;
    }

    void setData(const Vector<T>& data)
    {
        this->mData->setData(data);
    }

    void setItem(std::size_t idx, std::size_t jdx, std::size_t kdx, const T& value) override
    {
        this->mData->setItem(getOffset(idx, jdx, kdx), value);
    }

    std::size_t getDataSize() const override
    {
        return this->mData->getLength();
    }
};

template<typename T>
class SerializeableGrid : public Grid<T>
{
public:
    SerializeableGrid(std::size_t itemSize, const Bounds& bounds, std::size_t numPointsX, std::size_t numPointsY, std::size_t numPointsZ = 1)
        : Grid<T>(bounds, numPointsX, numPointsY, numPointsZ),
            mItemSize(itemSize)
    {
    }

    uint8_t getByte(std::size_t index)
    {
        const auto item_index = index/mItemSize;
        const auto item_internal_index = index % mItemSize;
        return this->mData->getItem(item_index).getByte(item_internal_index);
    }

    void toBuffer(uint8_t* buffer, std::size_t bufferMaxSize)
    {
        std::size_t count{0};
        std::size_t item_count{0};
        while(count < bufferMaxSize && item_count < this->mData->getLength())
        {
            const auto& item = this->mData->getItem(item_count);
            for(std::size_t idx = 0; idx< item.getSize(); idx++)
            {
                buffer[count] = item.getByte(idx);
                count++;
            }
            item_count++;
        }
    }

private:
    std::size_t mItemSize{1};
};
