#pragma once

#include "PathElement.h"

#include "Vector.h"
#include "Memory.h"
#include "String.h"

using PathElementPtr = Ptr<PathElement<2> >;

class GeometryPathFeature
{
public:
    void setLocation(Vector2 offset)
    {
        mOffset = offset;
    }

    void addElement(PathElementPtr element)
    {
        mElements.push_back(std::move(element));
    }

    const Vector2& getOffset() const
    {
        return mOffset;
    }

    const Vector<PathElementPtr>& getElements() const
    {
        return mElements;
    }

    PathOffsetType getPathOffsetType() const
    {
        return mOffsetType;
    }

    void setPathOffsetType(PathOffsetType offsetType)
    {
        mOffsetType = offsetType;
    }

private:
    Vector2 mOffset;
    PathOffsetType mOffsetType{PathOffsetType::RELATIVE_TO};
    Vector<PathElementPtr> mElements;
};
using GeometryPathFeaturePtr = Ptr<GeometryPathFeature>;

class GeometryPath : public AbstractGeometricItem
{
public:
    ~GeometryPath();

    void addFeature(GeometryPathFeaturePtr feature);

    Bounds getBounds() const override;

    Type getType() const override;

    const Vector<GeometryPathFeaturePtr>& getFeatures() const;

private:
    Vector<GeometryPathFeaturePtr> mFeatures;
};
