#pragma once

#include "Memory.h"
#include "String.h"
#include <functional>

class Socket
{
public:
    struct State
    {
        enum class ConnectStatus
        {
            UNSET,
            OK,
            FAILED
        };

        enum class BindStatus
        {
            UNSET,
            OK,
            FAILED
        };

        ConnectStatus mConnectStatus{ ConnectStatus::UNSET };
        BindStatus mBindStatus{ BindStatus::UNSET };
        String mErrorMessage;
        int mErrorCode{ 0 };
        String mBody;
    };

    Socket(const String& address, unsigned port);

    virtual ~Socket();

    String getAddress() const;

    unsigned getPort() const;

    const State& getState() const;

    virtual void respond(const String& message) = 0;

    virtual String recieve() = 0;

    virtual String send(const String& message) = 0;
protected:
    virtual void initialize() {};

    virtual void initializeForBind() {};

    virtual void doBind() {};

    virtual void doConnect() {};

    State mState;
    unsigned mPort{0};
    String mAddress;
};

using SocketPtr = Ptr<Socket>;
