#pragma once

#include "LatexSymbols.h"

#include "String.h"
#include "Memory.h"
#include "Vector.h"

class LatexMathExpression;
using LatexMathExpressionPtr = Ptr<LatexMathExpression>;

class LatexMathExpression
{
public:
    enum class Type
    {
        LINEAR,
        LEAF,
        FRACTION,
        ENCLOSING,
        SUBSCRIPT,
        SUPERSCRIPT
    };

    enum class LineState
    {
        NONE,
        IN_TAG_NAME,
        AWAITING_LBRACE,
        IN_TAG_BODY
    };

    LatexMathExpression(const String& expression);

    LatexMathExpression(Type type);

    ~LatexMathExpression();

    const Vector<LatexMathSymbol>& getSymbols() const;

    const Vector<LatexMathExpressionPtr>& getExpressions() const;

    const LatexMathSymbol& getLeftSymbol() const;

    const LatexMathSymbol& getRightSymbol() const;

    Type getType() const;

    void setLeftSymbol(const LatexMathSymbol& symbol);

    void setRightSymbol(const LatexMathSymbol& symbol);

    void setContent(Vector<LatexMathSymbol>& symbols);

    void setRawContent(const String& content);

    void onTagBodyChar(char c);

    void onFreeChar(char c);

    void onSpace();

    void onLBrace();

    void onRBrace();

    void onSuperscript();

    void onSubscript();

    void onLiteral();

    void onTagBody();

    void onFinishedTagName();

    void onCloseExpression();

    void onFracTag();

    void onEnclosingTag(LatexMathSymbol::Tag tag);

    void parse();

private:
    std::size_t mOpenBraceCount{ 0 };

    LineState mLineState{ LineState::NONE };
    String mWorkingString;
    String mRawBody;

    Type mWorkingType{ Type::LEAF };
    LatexMathExpression* mWorkingExpression{ nullptr };
    Type mType{ Type::LEAF };
    Vector<LatexMathSymbol> mWorkingSymbols;

    LatexMathSymbol mLeftSymbol;
    LatexMathSymbol mRightSymbol;

    Vector<LatexMathSymbol> mSymbols;
    Vector<LatexMathExpressionPtr> mExpressions;
};
