#pragma once

#include "Point.h"

#include "String.h"
#include "Vector.h"

class TextSpan
{
public:
	enum class Type
	{
		PLAIN,
		EQUATION
	};

	TextSpan(const String& content, Type type = Type::PLAIN)
		: mType(type),
		mContent(content)
	{

	}

private:
	Type mType{ Type::PLAIN };
	String mContent;
};

class PlotCaption
{
public:
	PlotCaption() = default;

	void setContent(const String& plainContent)
	{
		mContent = { plainContent };
	}

	void setContent(const Vector<TextSpan>& content)
	{
		mContent = content;
	}

private:
	Vector<TextSpan> mContent;
};

class PlotSeries
{
public:
	PlotSeries(const String& label = {})
		: mLabel(label)
	{

	}

	void setData(const Vector<Point2>& data)
	{
		mData = data;
	}

private:
	String mLabel;
	Vector<Point2> mData;
};

class Plot
{
public:
	Plot() = default;

	void setXAxisCaption(const String& caption)
	{
		mXAxisCaption.setContent(caption);
	}

	void setYAxisCaption(const String& caption)
	{
		mYAxisCaption.setContent(caption);
	}

	void addSeries(const PlotSeries& series)
	{
		mSeries.push_back(series);
	}

private:
	PlotCaption mXAxisCaption;
	PlotCaption mYAxisCaption;
	Vector<PlotSeries> mSeries;
};