#pragma once

#include "IFontEngine.h"
#include "FontGlyph.h"
#include "FontItem.h"

#include <wrl.h>

#include "Vector.h"
#include <unordered_map>

class GlyphOutlineGeometrySink;
class OffScreenTextRenderer;

struct IDWriteFactory;
struct IDWriteTextFormat;
struct IDWriteTextLayout;
struct IDWriteFontFace;
struct IDWriteFontFamily;

class DirectWriteFontEngine : public IFontEngine
{
public:
    void initialize() override;

    void loadFontFace(const std::filesystem::path& fontFile, float penSize = 16) override {};

    void loadFontFaceFromName(const String& fontName, float penSize = 16);

    Ptr<FontGlyph> loadGlyph(uint32_t charCode) override;

    Ptr<GlyphRunOutlines> getGlyphRunOutlines(const Vector<uint32_t>& charCodes);

    IDWriteFontFamily* getSystemFontFamily(const String& fontName);

    bool loadSystemFontFamily(const String& fontName);

    IDWriteFontFace* getDefaultFontFace(const String& fontName);

    bool loadDefaultFontFace(const String& fontName);

    double getHorizontalAdvance(const FontItem& fontItem, const String& content) override;

private:
    void initializeOffScreenRenderer();

    bool mIsValid{ false };

    std::unordered_map<String, Microsoft::WRL::ComPtr<IDWriteFontFace> > mDefaultFontFaces;
    std::unordered_map<String, Microsoft::WRL::ComPtr<IDWriteFontFamily> > mSystemFontFamilies;

    Microsoft::WRL::ComPtr<IDWriteFactory> mDWriteFactory;
    Microsoft::WRL::ComPtr<IDWriteTextFormat> mTextFormat;
    Microsoft::WRL::ComPtr<IDWriteTextLayout> mTextLayout;
    Microsoft::WRL::ComPtr<OffScreenTextRenderer> mOffScreenRenderer;
};