#pragma once

#include "Memory.h"
#include "Vector.h"

class AbstractPainter;
class DrawingSurface;
class FontsManager;

enum class DrawingMode
{
    RASTER,
    GRAPH
};

class DrawingContext
{
public:
    DrawingContext(DrawingSurface* surface, FontsManager* fontsManager = nullptr, DrawingMode requestedDrawingMode = DrawingMode::GRAPH);

    static Ptr<DrawingContext> Create(DrawingSurface* surface, FontsManager* fontsManager, DrawingMode requestedDrawingMode);

    DrawingSurface* getSurface() const;

    FontsManager* getFontsManager() const;

    void paint();

    AbstractPainter* getPainter() const;

private:
    DrawingMode mDrawingMode;
    FontsManager* mFontsManager{nullptr};
    DrawingSurface* mSurface{nullptr};

    Ptr<AbstractPainter> mPainter;
};

using DrawingContextPtr = Ptr<DrawingContext>;
