#pragma once

#include <wrl.h>
#include <d3d12.h>

#include "Memory.h"
#include "Vector.h"

class DrawingContext;
class DirectXShaderProgram;
class DirectXMesh;
class DirectXInterface;

class DirectXMeshPainter
{
public:
    DirectXMeshPainter();

    ID3D12PipelineState* getPipelineState() const;
    ID3D12RootSignature* getRootSignature() const;

    void initializeD3d();

    void setDxInterface(DirectXInterface* dxInterface);

    void updateBuffers(DrawingContext* context);
    void updateCommandList(ID3D12GraphicsCommandList* commandList);
private:
    void initializeShader();
    void createRootSignature();
    void createPipelineStateObject();

    DirectXInterface* mDxInterface{ nullptr };

    Vector<Ptr<DirectXMesh> > mDxMeshes;

    Microsoft::WRL::ComPtr<ID3D12RootSignature> mRootSignature;
    Microsoft::WRL::ComPtr<ID3D12PipelineState> mPipelineState;

    Ptr<DirectXShaderProgram> mShaderProgram;
};