#pragma once

#include "Point.h"

#include "Memory.h"
#include "Vector.h"
#include <unordered_map>
#include "String.h"

class Edge;

class AbstractFace
{
public:

    enum class Orientation
    {
        CW,
        CCW
    };

    AbstractFace(std::size_t id=0);

    virtual ~AbstractFace();

    virtual Vector<std::size_t> getNodeIds() const = 0;

    void addVectorAttribute(const String& tag, const Vector<double>& values);

    Vector<double> getVectorAttribute(const String& tag) const;

    virtual std::size_t getNumNodes() const = 0;

    virtual void associateWidthEdges() = 0;

    virtual Vector<std::size_t> getEdgeIds() const = 0;

    virtual Vector<Point3> getNodeLocations(Orientation orientation = Orientation::CCW) const = 0;

    virtual void replaceEdge(Edge* original, Edge* replacement) = 0;

    void setIndex(std::size_t idx);

protected:
    std::size_t mId{0};
    std::unordered_map<String, Vector<double> > mVectorAttributes;
};
