#pragma once

#include "Point.h"
#include "Node.h"
#include "Transform.h"

#include "Memory.h"
#include "Vector.h"

using NodePtr = Ptr<Node>;
using VecNodes = Vector<NodePtr>;

class AbstractMesh
{
public:
    enum class MeshType
    {
        LINE,
        TRI,
        QUAD
    };

    virtual ~AbstractMesh();

    void addVectorAttribute(const String& tag, const Vector<double>& values);

    void addConstantNodeVectorAttribute(const String& tag, const Vector<double>& values);

    template<typename T>
    Vector<T> getVerticesFlat(T scaleX = 1.0, T scaleY = 1.0, T scaleZ = 1.0) const
    {
        Vector<T> ret(3*mNodes.size());
        for(std::size_t idx = 0; idx<mNodes.size(); idx++)
        {
            auto node = mNodes[idx].get();
            ret[3*idx] = node->getPoint().getX()/scaleX;
            ret[3*idx + 1] = node->getPoint().getY()/scaleY;
            ret[3*idx + 2] = node->getPoint().getZ()/scaleZ;
        }
        return ret;
    }

    virtual Ptr<AbstractMesh> copy() const = 0;

    Vector<Vector<double> > getNodeVectorAttributes(const String& tag);

    Vector<double> getVectorAttribute(const String& tag) const;

    virtual MeshType getType() const = 0;

    bool hasVectorAttribute(const String& tag) const;

    void scale(double scaleX, double scaleY);

    void translate(const Point3& offset);

    void translate(double offsetX, double offsetY, double offsetZ = 0.0);

    void transform(const Transform& transform);

    unsigned getNumNodes() const;

    const VecNodes& getNodes() const;

protected:
    std::unordered_map<String, Vector<double> > mVectorAttributes;
    VecNodes mNodes;
};
