#pragma once

#include "Point.h"
#include "Transform.h"

#include <unordered_map>
#include "String.h"
#include "Vector.h"

class Node
{
public:

    enum class State
    {
        HEALTHY,
        DIRTY
    };

    Node(const Point3& p, std::size_t index = 0);

    static Ptr<Node> Create(const Point3& p, std::size_t index = 0);

    ~Node();

    void apply(const Transform& transform);

    void associateEdge(std::size_t edgeId);

    void associateFace(std::size_t faceId);

    void addVectorAttribute(const String& tag, const Vector<double>& values);

    void clearConnectivity();

    std::size_t getIndex() const;

    std::size_t getNumConnectedEdges() const;

    std::size_t getNumConnectedFaces() const;

    std::size_t getConnectedEdgeId(std::size_t idx) const;

    std::size_t getConnectedFaceId(std::size_t idx) const;

    State getState() const;

    const Point3& getPoint() const;

    Vector<double> getVectorAttribute(const String& tag) const;

    bool isCoincident(Node* node) const;

    void setState(State state);

    void setIndex(std::size_t idx);

    void updateIndex(std::size_t index);

private:
    std::unordered_map<String, Vector<double> > mVectorAttributes;
    std::size_t mIndex{0};
    Point3 mPoint;

    Vector<std::size_t> mAssociatedEdgeIds;
    Vector<std::size_t> mAssociatedFaceIds;

    State mState{State::HEALTHY};
};
