#pragma once

#include "SceneItem.h"
#include "Transform.h"

#include "Memory.h"
#include "String.h"
#include "Vector.h"

struct SceneInfo;

class AbstractVisualNode
{
public:
    AbstractVisualNode(const Transform& transform = {}, const String& name = {});

    virtual ~AbstractVisualNode();

    void addChild(AbstractVisualNode* child);

    virtual SceneItem* getSceneItem(std::size_t idx) const;

    virtual std::size_t getNumSceneItems() const;

    const Vector<AbstractVisualNode*>& getChildren() const;

    const String& getName() const;

    bool getIsVisible() const;

    const Transform& getTransform() const;

    void syncChildren(const Vector<AbstractVisualNode*>& children);

    void setIsVisible(bool isVisible);

    void setName(const String& name);

    virtual void setTransform(const Transform& transform);

    virtual void update(SceneInfo* sceneInfo);

protected:
    Transform mTransform;
    Vector<Ptr<SceneItem> > mSceneItems;

    Vector<AbstractVisualNode*> mChildren;

    bool mIsVisible{true};
    bool mTransformIsDirty{true};
    String mName;
};
