#pragma once

#include "DrawingSurface.h"
#include "DisplayState.h"
#include "DrawingContext.h"
#include "Widget.h"

#include "Memory.h"
#include "Vector.h"
#include <cstdint>
#include "String.h"
#include <functional>

class PaintEvent;
class MouseEvent;
class KeyboardEvent;

class DrawingContext;
class FontsManager;
enum class DrawingMode;

using WidgetPtr = Ptr<Widget>;
class IPlatformWindow;
using IPlatformWindowPtr = Ptr<IPlatformWindow>;

namespace mt
{

class Screen;

class Window : public DrawingSurface
{

public:
    using onPopupFunc = std::function<void(mt::Window* parent, Ptr<Widget> popup)>;

    Window(DrawingMode drawingMode = DrawingMode::GRAPH, FontsManager* fontsManager = nullptr);

    ~Window();

    static Ptr<Window> Create(DrawingMode drawingMode = DrawingMode::GRAPH, FontsManager* fontsManager = nullptr);

    void addPopup(Ptr<Widget> popupWidget);

    void clear();

    void clearPlatformWindow();

    void doPaint(mt::Screen* screen);

    IPlatformWindow* getPlatformWindow() const;

    const String& getTitle() const;

    Window* getParent() const;

    DrawingContext* getDrawingContent() const;

    WindowState::Display getDisplayState() const;

    bool getIsSizingOnGoing() const;

    bool isDirty() const;

    void map();

    void onPaint(const PaintEvent* event);

    void onMouseEvent(const MouseEvent* event);

    void onResize();

    void onKeyboardEvent(const KeyboardEvent* event);

    void setPlatformWindow(IPlatformWindowPtr window);

    void setPopupHandler(onPopupFunc func);

    void setParent(Window* parent);

    void show();

    void setIsSizingOnGoing(bool sizingOngoing);

    void setSize(unsigned width, unsigned height, bool triggerResize = true) override;

    void setTitle(const String& title);

    void setWidget(WidgetPtr widget);

    void setDisplayState(WindowState::Display state);

private:
    WidgetPtr mWidget {nullptr};
    String mTitle;
    IPlatformWindowPtr mPlatformWindow {nullptr};
    Ptr<DrawingContext> mDrawingContext;

    onPopupFunc mPopupFunc;
    Window* mParent{nullptr};

    bool mSizingOngoing{ false };
    WindowState::Display mDisplayState{ WindowState::Display::NORMAL };
};
}

using WindowUPtr = Ptr<mt::Window>;
