#pragma once

#include "Vector.h"
#include "Memory.h"

#include "Keyboard.h"
#include "Screen.h"

#include "KeyboardEvent.h"
#include "PaintEvent.h"
#include "MouseEvent.h"
#include "UiEvent.h"

#include "AbstractApp.h"

#include "EventManager.h"
#include "FontsManager.h"
#include "WindowManager.h"
#include "ThemeManager.h"

class AbstractDesktopApp;

class DesktopManager
{
public:
    DesktopManager(AbstractDesktopApp* application);

    ~DesktopManager();

    static Ptr<DesktopManager> Create(AbstractDesktopApp* application);

    void setWindowManager(WindowManagerUPtr windowManager);

    AbstractApp* getMainApp() const;

    WindowManager* getWindowManager() const;

    Keyboard* getKeyboard() const;

    FontsManager* getFontsManager() const;

    ThemeManager* getThemeManager() const;

    void addScreen(ScreenPtr screen);

    mt::Screen* getDefaultScreen() const;

    void setKeyboard(KeyboardUPtr keyboard);

    void onUiEvent(UiEventUPtr event);

    void onKeyboardEvent(const KeyboardEvent* keyboardEvent);

    void onMouseEvent(const MouseEvent* mouseEvent);

    void onPaintEvent(const PaintEvent* paintEvent);

    void clearEvents();

    void onLoopIteration();

    void addWindow(mt::Window* window);

private:
    Vector<ScreenPtr> mScreens;
    KeyboardUPtr mKeyboard;

    FontsManagerPtr mFontsManager;
    WindowManagerUPtr mWindowManager;
    EventManagerUPtr mEventManager;
    Ptr<ThemeManager> mThemeManager;

    AbstractDesktopApp* mUiApplication;
};

using DesktopManagerUPtr = Ptr<DesktopManager>;
