#pragma once

#include "String.h"

class MarkdownCustomElementContext
{
public:
    MarkdownCustomElementContext(const String& delimiter)
        : mDelimiter(delimiter)
    {

    }

    virtual ~MarkdownCustomElementContext() = default;

    void setReplacementDelimiters(const String& leftReplacement, const String& rightReplacement)
    {
        mReplacementLeftDelimiter = leftReplacement;
        mReplacementRightDelimiter = rightReplacement;
        if (mReplacementRightDelimiter.empty())
        {
            mReplacementRightDelimiter = mReplacementLeftDelimiter;
        }
    }

    const String& getDelimiter() const
    {
        return mDelimiter;
    }

    const String& getOutputLeftDelimiter() const
    {
        if (!mReplacementLeftDelimiter.empty())
        {
            return mReplacementLeftDelimiter;
        }
        else
        {
            return mDelimiter;
        }
    }

    const String& getOutputRightDelimiter() const
    {
        if (!mReplacementRightDelimiter.empty())
        {
            return mReplacementRightDelimiter;
        }
        else
        {
            return mDelimiter;
        }
    }

private:
    String mReplacementLeftDelimiter;
    String mReplacementRightDelimiter;
    String mDelimiter;
};

class MarkdownCustomInlineElementContext : public MarkdownCustomElementContext
{
public:
    MarkdownCustomInlineElementContext(const String& delimiter)
        : MarkdownCustomElementContext(delimiter)
    {

    }
};

class MarkdownCustomMultilineElementContext : public MarkdownCustomElementContext
{
public:
    MarkdownCustomMultilineElementContext(const String& delimiter)
        : MarkdownCustomElementContext(delimiter)
    {

    }
};
